__doc__="""
Zope installation utility

You must have adminstrative priveleges to
successfully run this installation!!

Usage

   Zope -i [options]

Options

   -c   -- Supply the cgi directory where the pcgi
           information file will be generated. If not
           specified, this file will be generated in
           the root directory of your installation and
           can be moved into a cgi directory on your
           web server at a later time.

   -g   -- Supply the name of the unix group to which
           the user that runs your web server belongs.
           If not specified, the installer will attempt
           to determine the group itself. If no group
           is specified and the installer is unable to
           determine the group, the install will fail.

   -h   -- Show command summary

   -s   -- Provide superuser name and password in the format
           name:password. This is REQUIRED for a first time
           installation.

   -u   -- Supply the name of the unix user used to
           run your web server. If not specified, this
           defaults to the userid of the current user,
           or 'nobody' is the current user is root.

   -r   -- This is an RPM (or RPM-like) installation.

           o Get the installation name from the directory
	     name where installe.

           o Use the installation name as the superuser name and use a
             blank superuser password.  This means that the
             administrator *must* edit access or configure their web
             server to authenticate this name.

	   o Create a link for the executable using the name, so that
             ps listings show the installation name.

	   o Write the resource file accordingly.
             


"""
__version__='$Revision: 1.20 $'[11:-2]



import sys,os,getopt,pwd,string,regsub


def error(s):
    sys.stderr.write('Installation FAILED!\n')
    sys.stderr.write('%s\n' % s)
    sys.exit(-1)


def status(s):
    sys.stdout.write('%s\n' % s)


def setup_var(rootdir, user, group):
    dir=os.path.join(rootdir, 'var')
    if (not os.path.isdir(dir)):
        status('Setting up var directory...')
	try:    os.mkdir(dir,0770)
	except: error('Failed to create directory %s!' % dir)

    # Create default db if none exists
    db=os.path.join(dir, 'Data.bbb')
    if (not os.path.exists(db)):
        status('Setting up default data...')
        fn=os.path.join(rootdir, 'inst/Data.bbb')
        os.system('cp %s %s' % (fn, db))
        db=db + '.index'
        os.system('rm -f %s' % db)
        
    if os.system('chown -R %s %s' % (user, dir)) != 0:
	error('Failed to chown %s to %s!' % (dir, user))
    if os.system('chgrp -R %s %s' % (group, dir)) != 0:
	error('Failed to chgrp %s to %s!' % (dir, group))
    if os.system('chmod -R 0755 %s' % dir) != 0:
	error('Failed to chmod %s to 0755!' % dir)


def setup_pcgi(rootdir,user,group,cgidir,rpm):

    info="""#!%(root)s/pcgi/bin/pcgi-wrapper
PCGI_NAME=Main
PCGI_MODULE_PATH=%(root)s/lib/python/Main.py
PCGI_PUBLISHER=%(root)s/pcgi/pcgi_publisher.py
PCGI_EXE=%(root)s/bin/python
PCGI_SOCKET_FILE=%(root)s/var/pcgi.soc
PCGI_PID_FILE=%(root)s/var/pcgi.pid
PCGI_ERROR_LOG=%(root)s/var/pcgi.log
PCGI_DISPLAY_ERRORS=1
PYTHONHOME=%(root)s
BOBO_REALM=%(name)s
BOBO_DEBUG_MODE=1
""" % {'root': rootdir, 'name':'Zope'}

    fn=os.path.join(cgidir, 'Zope.cgi')
    status('Creating pcgi info file: %s' % fn)
    try:    file=open(fn, 'w')
    except: error('Failed to create file: %s' % fn)
    file.write(info)
    file.close()
    if os.system('chown %s %s' % (user, fn)) != 0:
        error('Unable to chown %s to %s' % (fn, user))
    if os.system('chgrp %s %s' % (group, fn)) != 0:
        error('Unable to chgrp %s to %s' % (fn, group))
    if os.system('chmod 0755 %s' % fn) != 0:
        error('Unable to chmod %s to 0755' % fn)
    status('Created pcgi info file successfully')



def setup_access(rootdir, name, password):
    fname=os.path.join(rootdir, 'access')
    pfile=open(fname, 'w')
    pfile.write('%s:%s' % (name, password))
    pfile.close()

    if os.system('chmod 0644 %s' % fname) != 0:
        error('Unable to chmod %s to 0644' % fname)

rpm_blather="""
Zope has been installed in %(dir)s.

To run Zope with the included Python web server, 
run:

  %(dir)s/start 

In your Web browser, visit the URL http://localhost:9673/welcome to
begin evaluation.  When you are prompted for a username and password,
please use "Zope" for a username and "%(pw)s" for a password.  
If you forget or wish to change the user name or password, you should
edit the file:

  %(dir)s/access.

To shutdown the Zope servers, 
please run:

  %(dir)s/stop 


If you wish to use Zope with an existing web server see doc/WEBSERVER.txt
The PCGI info file that you will need has been written to:

  %(dir)s/Zope.cgi

"""

def setup_start_stop(rootdir):
    open('start','w').write("""#! /bin/sh
PYTHONHOME=%(rootdir)s
export PYTHONHOME
cd %(rootdir)s
bin/python serve.py -Pvar/serve.pid 1> var/serve.log 2> var/serve.errors &
""" % vars())
    open('stop','w').write("""#! /bin/sh
kill `cat %(rootdir)s/var/serve.pid`
""" % vars())
    os.system("chmod +x start stop")

def main():
    rootdir=os.getcwd()
    cgidir =rootdir
    user=pwd.getpwuid(os.getuid())[0] # install as user's id by default
    if user=="root": user="nobody"    # or 'nobody' if run by root
    group=None
    password=None

    try:    options,args=getopt.getopt(sys.argv[1:], 'c:g:ihs:u:r')
    except: error(__doc__)

    rpm=name='Zope'
    for n,v in options:
	if n=='-c': cgidir=string.strip(v)
	if n=='-g': group =string.strip(v)
	if n=='-s':
	    try:    name, password=tuple(string.split(string.strip(v), ':'))
	    except: error('Superuser name and password must be in ' \
			  ' the format username:password')
	if n=='-u': user  =string.strip(v)
	if n=='-h':
	    status(__doc__)
	    sys.exit(0)

    if not name:
	error('You MUST specify a superuser name and password for this ' \
	      'installation!\n  Use -sUsername:Password')

    if password is None:
	import random
	password=string.join(
	    map(lambda ig, c=random.choice,
                l=string.letters+string.digits+'_-':
		c(l), range(8)),'')

    setup_access(rootdir, name, password)

    if group is None:
        try:    group=str(pwd.getpwnam(user)[3])
	except: error('UID for the user %s was not found.\n' % user)

    setup_var(rootdir, user, group)
    setup_pcgi(rootdir, user, group, cgidir,rpm)
    setup_start_stop(rootdir)
    status('Installation completed successfully.')

    if rpm: print rpm_blather % {
	'dir': rootdir,
	'pw': password,
	}

    sys.exit(0)


main()

